'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _child_process = require('child_process');

var _bluebird = require('bluebird');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _cliColor = require('cli-color');

var _cliColor2 = _interopRequireDefault(_cliColor);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _gitExec = require('git-exec');

var _gitExec2 = _interopRequireDefault(_gitExec);

var _which = require('which');

var _which2 = _interopRequireDefault(_which);

var _ = require('../');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var npm = _which2.default.sync('npm');

var singleton = Symbol();
var singletonEnforcer = Symbol();

var Plugins = function () {
  function Plugins(enforcer) {
    _classCallCheck(this, Plugins);

    if (enforcer != singletonEnforcer) throw 'Cannot construct Plugins singleton';
    this._plugins = [];
    this.fn = [];
    var pluginsDir = _path2.default.join(_.config.root, _.config.plugins.url);
    try {
      var directoryPlugins = _fsExtra2.default.lstatSync(pluginsDir);
      if (directoryPlugins.isDirectory()) {

        this._plugins = _.coreUtils.file.getFoldersSync(pluginsDir, false);
        Array.prototype.forEach.call(this._plugins, function (plugin) {
          var name = plugin.path.replace(pluginsDir + '/', '');
          // has hooks
          var plugHooks = _path2.default.join(plugin.path, _.config.hooks.url);
          try {
            var directoryHook = _fsExtra2.default.lstatSync(plugHooks);
            if (directoryHook.isDirectory()) {
              var plugHooksFile = _path2.default.join(plugHooks, 'hooks.js');
              var h = require(plugHooksFile);
              plugin.hooks = h.default;
            } else {
              plugin.hooks = null;
            }
          } catch (e) {
            plugin.hooks = null;
          }

          // has partials
          var plugPartials = _path2.default.join(plugin.path, _.config.pluginsPartials);
          try {
            var directoryPartials = _fsExtra2.default.lstatSync(plugPartials);
            if (directoryPartials.isDirectory()) {
              plugin.partials = plugPartials;
            } else {
              plugin.partials = null;
            }
          } catch (e) {
            plugin.partials = null;
          }

          // has templates
          var plugTemplates = _path2.default.join(plugin.path, _.config.templates.url);
          try {
            var directoryTemplates = _fsExtra2.default.lstatSync(plugTemplates);
            if (directoryTemplates.isDirectory()) {
              plugin.templates = plugTemplates;
            } else {
              plugin.templates = null;
            }
          } catch (e) {
            plugin.templates = null;
          }

          // has process
          var plugProcess = _path2.default.join(plugin.path, 'process');
          try {
            var directoryProcess = _fsExtra2.default.lstatSync(plugProcess);
            if (directoryProcess.isDirectory()) {
              plugin.process = {};
              var processFiles = _.coreUtils.file.getFilesSync(plugProcess, false);
              Array.prototype.forEach.call(processFiles, function (processFile) {
                plugin.process[_path2.default.basename(processFile, '.js')] = processFile;
              });
            } else {
              plugin.process = null;
            }
          } catch (e) {
            plugin.process = null;
          }

          // has routes
          var plugRoutes = _path2.default.join(plugin.path, 'routes');
          try {
            var directoryRoute = _fsExtra2.default.lstatSync(plugRoutes);
            if (directoryRoute.isDirectory()) {
              plugin.routes = {};

              var gets = _path2.default.join(plugRoutes, 'get');
              try {
                var directoryGets = _fsExtra2.default.lstatSync(gets);
                if (directoryGets.isDirectory()) {
                  (function () {
                    var routesGet = [];
                    var routePaths = _.coreUtils.file.getFilesSync(gets, false);
                    Array.prototype.forEach.call(routePaths, function (route) {
                      var pathUrl = '/abe/plugin/' + name + '/' + _path2.default.basename(route, '.js') + '*';
                      var routeObject = { 'path': route, 'routePath': pathUrl };
                      routesGet.push(routeObject);
                    });
                    plugin.routes.get = routesGet;
                  })();
                }
              } catch (e) {
                plugin.routes.get = null;
              }
              try {
                var posts = _path2.default.join(plugRoutes, 'post');
                var directoryPosts = _fsExtra2.default.lstatSync(gets);
                if (directoryPosts.isDirectory()) {
                  (function () {
                    var routesPost = [];
                    var routePaths = _.coreUtils.file.getFilesSync(posts, false);
                    Array.prototype.forEach.call(routePaths, function (route) {
                      var pathUrl = '/abe/plugin/' + name + '/' + _path2.default.basename(route, '.js') + '*';
                      var routeObject = { 'path': route, 'routePath': pathUrl };
                      routesPost.push(routeObject);
                    });
                    plugin.routes.post = routesPost;
                  })();
                }
              } catch (e) {
                plugin.routes.post = null;
              }
            } else {
              plugin.routes = null;
            }
          } catch (e) {
            plugin.routes = null;
          }
        });
      }
    } catch (e) {}
  }

  _createClass(Plugins, [{
    key: 'getProcess',
    value: function getProcess(fn) {
      var proc = null;
      if (typeof this._plugins !== 'undefined' && this._plugins !== null) {
        Array.prototype.forEach.call(this._plugins, function (plugin) {
          if (typeof plugin.process !== 'undefined' && plugin.process !== null && typeof plugin.process[fn] !== 'undefined' && plugin.process[fn] !== null) {
            proc = plugin.process[fn];
          }
        });
      }

      return proc;
    }
  }, {
    key: 'hooks',
    value: function hooks() {
      var _this = this;

      if (arguments.length > 0) {
        var args = [].slice.call(arguments);
        var fn = args.shift();

        if (this._plugins != null) {
          Array.prototype.forEach.call(this._plugins, function (plugin) {
            if (plugin.hooks != null && plugin.hooks[fn] != null) {
              args[0] = plugin.hooks[fn].apply(_this, args);
            }
          });
        }
      } else {
        args = [''];
      }

      return args[0];
    }
  }, {
    key: 'getHooks',
    value: function getHooks() {
      return this._plugins;
    }
  }, {
    key: 'getPartials',
    value: function getPartials() {
      var partials = [];
      Array.prototype.forEach.call(this._plugins, function (plugin) {
        if (typeof plugin.partials !== 'undefined' && plugin.partials !== null) {
          partials.push(plugin.partials);
        }
      });

      return partials;
    }
  }, {
    key: 'getRoutes',
    value: function getRoutes() {
      var routes = [];
      Array.prototype.forEach.call(this._plugins, function (plugin) {
        if (typeof plugin.routes !== 'undefined' && plugin.routes !== null) {
          routes = routes.concat(plugin.routes);
        }
      });

      return routes;
    }
  }, {
    key: 'add',
    value: function add(dir, plugin) {
      var p = new _bluebird.Promise(function (resolve) {

        var pluginName = plugin.split('/');
        pluginName = pluginName[pluginName.length - 1].split('.')[0];
        var pluginDir = _path2.default.join(dir, 'plugins', pluginName);

        try {
          _fsExtra2.default.statSync(pluginDir);
          console.log(_cliColor2.default.green('remove plugin'), pluginName);
          _fsExtra2.default.removeSync(pluginDir);
        } catch (e) {}
        console.log(_cliColor2.default.green('mkdir'), _cliColor2.default.green(pluginDir));
        (0, _mkdirp2.default)(pluginDir);

        _gitExec2.default.clone(plugin, pluginDir, function (repo) {
          if (repo !== null) {
            try {
              console.log(_cliColor2.default.green('cd'), _cliColor2.default.green(pluginDir));
              process.chdir(pluginDir);

              console.log(_cliColor2.default.green('spawn'), _cliColor2.default.cyan('npm install'));
              // const npmInstall = spawn('npm', ['install', pluginDir]);
              var npmInstall = (0, _child_process.spawn)(npm, ['install']);

              npmInstall.stdout.on('data', function (data) {
                var str = data.toString(),
                    lines = str.split(/(\r?\n)/g);
                for (var i = 0; i < lines.length; i++) {
                  console.log(str);
                }
              });

              npmInstall.stderr.on('data', function (data) {
                var str = data.toString(),
                    lines = str.split(/(\r?\n)/g);
                for (var i = 0; i < lines.length; i++) {
                  console.log(str);
                }
              });

              npmInstall.on('close', function (code) {
                console.log(_cliColor2.default.cyan('child process exited with code'), code);

                var json = {};
                var abeJson = dir + '/abe.json';

                try {
                  var stat = _fsExtra2.default.statSync(abeJson);
                  if (stat) {
                    json = _fsExtra2.default.readJsonSync(abeJson);
                  }
                } catch (e) {
                  console.log(e);
                  console.log(_cliColor2.default.cyan('no abe.json creating'), abeJson);
                }

                if (typeof json.dependencies === 'undefined' || json.dependencies === null) {
                  json.dependencies = [plugin];
                } else {
                  var found = false;
                  Array.prototype.forEach.call(json.dependencies, function (plugged) {
                    if (plugin === plugged) {
                      found = true;
                    }
                  });
                  if (!found) {
                    json.dependencies.push(plugin);
                  }
                }

                _fsExtra2.default.writeJsonSync(abeJson, json, { space: 2, encoding: 'utf-8' });
                resolve();
              });

              npmInstall.on('error', function (err) {
                console.log(_cliColor2.default.red('cannot install npm dependencies for'), pluginDir);
                console.log(err);
                resolve(err);
              });
            } catch (err) {
              console.log(_cliColor2.default.cyan('chdir'), err);
              resolve(err);
            }
          } else {
            console.log(_cliColor2.default.red('clone error'));
            resolve(err);
          }
        });
      });

      return p;
    }
  }], [{
    key: 'instance',
    get: function get() {
      if (!this[singleton]) {
        this[singleton] = new Plugins(singletonEnforcer);
      }
      return this[singleton];
    }
  }]);

  return Plugins;
}();

exports.default = Plugins;