#!/usr/bin/env node
'use strict';

var _bluebird = require('bluebird');

var _Create = require('./cli/cms/Create');

var _Create2 = _interopRequireDefault(_Create);

var _plugins = require('./cli/extend/plugins');

var _plugins2 = _interopRequireDefault(_plugins);

var _child_process = require('child_process');

var _childProcessPromise = require('child-process-promise');

var _childProcessPromise2 = _interopRequireDefault(_childProcessPromise);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _package = require('../package');

var _package2 = _interopRequireDefault(_package);

var _pm = require('pm2');

var _pm2 = _interopRequireDefault(_pm);

var _cliColor = require('cli-color');

var _cliColor2 = _interopRequireDefault(_cliColor);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_commander2.default.version(_package2.default.version).option('-v, --version', 'version').option('-p, --port <port>', 'Port on which to listen to (defaults to 8000)', parseInt).option('-i, --interactive', 'open in browser').option('-N, --pname <pname>', 'pm2 server name').option('-f, --folder <folder>', '--folder draft|sites').option('-t, --type <type>', '--type draft|other').option('-d, --destination <destination>', '--destination folder').option('generate-posts, --path=<folder> --destination=<folder> --status=<status>', 'save only <status> posts into <destination> from <path> folder').parse(process.argv);

var userArgs = process.argv.slice(2);
var create = new _Create2.default();
var port = _commander2.default.port;
var interactive = _commander2.default.interactive;

var vPos = process.argv.indexOf('-v');
if (vPos > -1) {
  process.argv[vPos] = '-V';
}

if (typeof userArgs[0] !== 'undefined' && userArgs[0] !== null) {
  var dir;
  var command;
  var cp;
  var plugin;

  switch (userArgs[0]) {
    case 'build':
      dir = process.cwd();
      process.chdir(__dirname + '/../');
      command = 'ROOT=' + dir + ' FOLDER=' + (_commander2.default.folder ? _commander2.default.folder : 'draft') + ' DEST=' + (_commander2.default.destination ? _commander2.default.destination : 'tmp') + ' FLOW=' + (_commander2.default.type ? _commander2.default.type : 'draft') + ' npm run build:folder';
      console.log('command : ' + command);
      _childProcessPromise2.default.exec(command).then(function (result) {
        var stdout = result.stdout;
        var stderr = result.stderr;
        if (stdout) {
          console.log('stdout: ', stdout);
        }
        if (stderr) {
          console.log('stderr: ', stderr);
        }
      }).fail(function (err) {
        console.error('ERROR: ', err);
      }).progress(function () {});
      break;
    case 'create':
      dir = userArgs[1];
      if (process.env.ROOT) {
        dir = process.env.ROOT + userArgs[1];
      }
      if (typeof dir !== 'undefined' && dir !== null) {
        create.init(dir);
      } else {
        console.error('Error: no project path specified');
      }
      break;
    case 'serve':
      dir = process.cwd();
      if (process.env.ROOT) {
        dir = process.env.ROOT;
      }
      var environment = process.env;
      environment.ROOT = dir;
      if (typeof port !== 'undefined' && port !== null) {
        environment.PORT = port;
      }
      command = 'node --harmony ./dist/server/index.js';
      // if (interactive) command = 'OPENURL=1 ' + command
      process.chdir(__dirname + '/../');
      console.log('website started : ' + dir + (port ? ' on port :' + port : ''));
      cp = (0, _child_process.exec)(command, {
        env: environment
      }, function (err, out, code) {
        if (err instanceof Error) throw err;
        process.stderr.write(err);
        process.stdout.write(out);
        process.exit(code);
      });
      cp.stderr.pipe(process.stderr);
      cp.stdout.pipe(process.stdout);
      break;
    case 'list':
      dir = process.cwd();
      dir = dir.replace(/\/$/, '');
      _pm2.default.connect(function (err) {
        if (err instanceof Error) throw err;

        _pm2.default.list(function (err, list) {
          if (err instanceof Error) throw err;
          Array.prototype.forEach.call(list, function (item) {
            console.log('[ pm2 ]', '{', '"pid":', item.pid + ',', '"process":', '"' + item.name + '"', '}');
          });
          process.exit(0);
        });
      });
      break;
    case 'servenodemon':
      dir = process.cwd();
      command = 'ROOT=' + dir + ' npm run startdev --node-args="--debug"';
      if (interactive) command = 'OPENURL=1 ' + command;
      if (port) command = 'PORT=' + port + ' ' + command;
      process.chdir(__dirname + '/../');
      console.log('website started : ' + dir + (port ? ' on port :' + port : ''));
      cp = (0, _child_process.exec)(command, function (err, out, code) {
        if (err instanceof Error) {
          throw err;
        }
        process.exit(code);
      });
      cp.stderr.pipe(process.stderr);
      cp.stdout.pipe(process.stdout);
      break;
    case 'generate-posts':
      dir = process.cwd();
      if (process.env.ROOT) {
        dir = process.env.ROOT.replace(/\/$/, '');
      }
      process.env.DEBUG = 'generate-posts:*';
      var generateArgs = ['--harmony', __dirname + '/cli/process/generate-posts.js', 'ABE_WEBSITE=' + dir];
      var isHelp = false;
      Array.prototype.forEach.call(userArgs, function (arg) {
        if (arg.indexOf('--path=') > -1) {
          generateArgs.push('ABE_PATH=' + arg.split('=')[1]);
        } else if (arg.indexOf('--destination=') > -1) {
          generateArgs.push('ABE_DESTINATION=' + arg.split('=')[1]);
        } else if (arg.indexOf('--status=') > -1) {
          generateArgs.push('ABE_STATUS=' + arg.split('=')[1]);
        }
      });

      var generate = (0, _child_process.spawn)('node', generateArgs, { shell: true, stdio: 'inherit' });

      generate.on('close', function (code) {
        console.log(_cliColor2.default.cyan('child process exited with code') + ' ' + code);
        process.exit(0);
      });

      break;
    case 'install':
      dir = process.cwd();
      plugin = userArgs[1];
      if (process.env.ROOT) {
        dir = process.env.ROOT.replace(/\/$/, '');
      }

      var json = {};
      var abeJson = dir + '/abe.json';

      try {
        var stat = _fsExtra2.default.statSync(abeJson);
        if (stat) {
          json = _fsExtra2.default.readJsonSync(abeJson);
        }
      } catch (e) {
        console.log(_cliColor2.default.cyan('no config'), abeJson);
      }

      var ps = [];
      if (typeof json.dependencies !== 'undefined' || json.dependencies !== null) {
        Array.prototype.forEach.call(json.dependencies, function (plugged) {
          ps.push(_plugins2.default.instance.add(dir, plugged));
        });
      }

      _bluebird.Promise.all(ps).then(function () {
        process.exit(0);
      });
      break;
    case 'add':
      dir = process.cwd();
      plugin = userArgs[1];
      if (process.env.ROOT) {
        dir = process.env.ROOT.replace(/\/$/, '');
      }

      if (typeof dir !== 'undefined' && dir !== null) {
        if (typeof plugin !== 'undefined' && plugin !== null) {
          _plugins2.default.instance.add(dir, plugin).then(function () {
            process.exit(0);
          });
        } else {
          console.log(_cliColor2.default.red('Error: no project path specified'));
        }
      } else {
        console.log(_cliColor2.default.red('Error: no project path specified'));
      }
      break;
    default:
      console.log('Help: use `create` or `serve` command');
      break;
  }
} else {
  console.log('Help: use `create` or `serve` command');
}